/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
	 Vulkan Graphics API: 11 Steps

   Minimalistic examples to build upon and experiment with 
	 to develop an understanding of the underlying API

	   
/*
/****************************************************************/
/*
     About:
	 Simple graphical skinning demo using the Vulkan API.   
   Remember to include the binary shader files (or compile
   them if you only have the .vert/.frag) to the .spv files
   (see buildshaders.bat).

	 ''''
	 Modifications to the original book - code is modularized
	 into functions (i.e., shows what parameters go in/out).
	 ''''

*/


#include "setup.h"


int CALLBACK WinMain( HINSTANCE hInstance, 
					  HINSTANCE hPrevInstance, 
					  LPSTR		lpCmdLine, 
					  int		nCmdShow ) 
{
	// Step-1-
	int				width				= 800;
	int				height				= 600;
	HWND			windowHandle		= NULL;
	SetupWindow(width, height, &windowHandle);

	// Step-2-
	VkInstance		instance			= NULL;
	VkSurfaceKHR	surface				= NULL;
	SetupVulkanInstance(windowHandle,
		                &instance,
					    &surface);

	// Step-3-
	VkPhysicalDevice physicalDevice		= NULL;
  VkDevice	device					        = NULL;
	SetupPhysicalDevice(instance,
                        &physicalDevice,
                        &device);

	// Step-4-
	VkSwapchainKHR   swapChain			= NULL;					     
	VkImage*         presentImages		= NULL;					     
	VkImageView*     presentImageViews	= NULL;
	SetupSwapChain( device,
					physicalDevice,
					surface,
					&width,
					&height,
					&swapChain,			     
					&presentImages,				     
					&presentImageViews);

  // Step-5-
	VkRenderPass     renderPass			= NULL;
	VkFramebuffer*   frameBuffers		= NULL;
	SetupRenderPass(device,
					physicalDevice,
					width,
					height,
					presentImageViews,
					&renderPass, 
					&frameBuffers);

	// Step-6-
	VkCommandBuffer  commandBuffer      = NULL;
	SetupCommandBuffer(device,
					   physicalDevice,
					   &commandBuffer);
	

  Skin* skin = NULL;
  SetupGeometry(device,
                physicalDevice,
                width,
                height,
                renderPass,
					      &skin);

	// Step-8-
	VkShaderModule   vertShaderModule  = NULL;
	VkShaderModule   fragShaderModule  = NULL;
  VkShaderModule   vertShadowShaderModule  = NULL;
	VkShaderModule   fragShadowShaderModule  = NULL;
	//VkBuffer		     uniformbuffer            = NULL;
	//VkDeviceMemory	 uniformmemory            = NULL;
	SetupShaders(device,
						   physicalDevice,
						   width,
						   height,
						   &vertShaderModule,
						   &fragShaderModule);
						 //  &uniformbuffer,
						 //  &uniformmemory);

	// Step-9-
	VkDescriptorSet       descriptorSet			= NULL;
	VkDescriptorSetLayout descriptorSetLayout   = NULL;
	SetupDescriptors(device,
                   physicalDevice,
					         skin,
					         &descriptorSet,
					         &descriptorSetLayout);

	// Step-10-
	VkPipeline       pipeline          = NULL;
	VkPipelineLayout pipelineLayout    = NULL;
	SetupPipeline(device,
                width,
                height,
                skin,
                descriptorSetLayout,
                vertShaderModule,
                fragShaderModule,
                renderPass, 
                &pipeline,
                &pipelineLayout);




  // Step-11-
	MSG msg;
	while( true ) 
	{
		// Continually force the window to be
		// redrawn as long as no other Win32 
		// messages are pending
		PeekMessage( &msg, NULL, NULL, NULL, PM_REMOVE );
		if( msg.message == WM_QUIT ) 
		{
			break;
		} 
		else 
		{
			TranslateMessage( &msg ); 
			DispatchMessage( &msg );
		}


		RenderLoop( device,
          physicalDevice,
					width,
					height,
					skin,
					swapChain,
					commandBuffer,
					presentImages,
					frameBuffers,
					renderPass,
					descriptorSet,
					pipelineLayout,
					pipeline);

	}// End while(true)
	return 0;
}// End WinMain(..)

