/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***

     Vulkan Graphics API: 11 Steps

     Minimalistic examples to build upon and experiment with 
     to develop an understanding of the underlying API

/*
/****************************************************************/
/*
     About:
	 Self contained demo (i.e., each section encapsulated within
	 a function - see the parameter dependencies). 
	 Remember to include and compile the two binary shader
	 files (vertex and shader)

	 ''''
	 #define for enabling/disabling `DEPTH_BUFFER' - 
	 extension (additional code) is needed from the minimal
	 version in the original book to have a working
	 depth buffer (i.e., zbuffer).
	 ''''

*/


#pragma once


//#define _CRT_SECURE_NO_WARNINGS // use fopen(..)

#include <windows.h>
#include <math.h>

#include <vector>
using namespace std;

// For variable argument functions ,e.g., dprintf(..), 
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>  // vsprintf_s


#include "helpers.h" // simple vector/matrix classes

//class Mesh; // forward declaration

class Skin;

// Windows version of the Vulkan API
#define VK_USE_PLATFORM_WIN32_KHR

// this is if we do not declared prototypes, 
// and want to load dynamically!
//#define VK_NO_PROTOTYPES            
                                      
#include "vulkan.h"



#define DEPTH_BUFFER



// Win32 message callback
LRESULT CALLBACK WindowProc( HWND	  hwnd, 
                             UINT	  uMsg, 
                             WPARAM wParam, 
                             LPARAM lParam );


// Step-1-
void SetupWindow(int   width, 
                 int   height, 
                 HWND* outWindowHandle);




// Step-2-
void SetupVulkanInstance(HWND          windowHandle,
                         VkInstance*   outInstance,
                         VkSurfaceKHR* outSurface);





// Step-3-
void SetupPhysicalDevice(VkInstance         instance,
                         VkPhysicalDevice*  outPhysicalDevice,
                         VkDevice*          outDevice);




// Step-4-
void SetupSwapChain(VkDevice          device,
                    VkPhysicalDevice  physicalDevice,
                    VkSurfaceKHR      surface,
                    int*              outWidth,
                    int*              outHeight,
                    VkSwapchainKHR*   outSwapChain,
                    VkImage**         outPresentImages,
                    VkImageView**     outPresentImageViews);




// Step-5-
void SetupRenderPass(VkDevice          device,
                     VkPhysicalDevice  physicalDevice,
                     int               width,
                     int               height,
                     VkImageView*      presentImageViews,
                     VkRenderPass*     outRenderPass, 
                     VkFramebuffer**   outFrameBuffers);



// Step-6-
void SetupCommandBuffer(VkDevice          device,
                        VkPhysicalDevice  physicalDevice,
                        VkCommandBuffer*  outCommandBuffer);



// Step-7-
void SetupGeometry(VkDevice         device,
                   VkPhysicalDevice physicalDevice,
                   int              width,
                   int              height,
                   VkRenderPass     renderPass,
                   Skin**           skin);




// Step-8-
void SetupShaders(VkDevice          device,
                  VkPhysicalDevice  physicalDevice,
                  int               width,
                  int               height,
                  VkShaderModule*   outVertShaderModule,
                  VkShaderModule*   outFragShaderModule);


// dynamically update the uniform buffer (e.g., projection/view..)
//void UpdateUniformBuffers(VkDevice          device,
//                          VkPhysicalDevice  physicalDevice,
//                          int               width,
//                          int               height,
//                          VkDeviceMemory    memory);


// Step-9-
void SetupDescriptors(VkDevice               device,
                      VkPhysicalDevice       physicalDevice,
                      Skin*                  skin,
                      VkDescriptorSet*       outDescriptorSet,
                      VkDescriptorSetLayout* outDescriptorSetLayout);






// Step-10-
void SetupPipeline(	VkDevice               device,
                    int                    width,
                    int                    height,
                    Skin*                  skin,
                    VkDescriptorSetLayout  descriptorSetLayout,
                    VkShaderModule         vertShaderModule,
                    VkShaderModule         fragShaderModule,
                    VkRenderPass           renderPass, 
                    VkPipeline*            outPipeline,
                    VkPipelineLayout*      outPipelineLayout);




// Step-11-
void RenderLoop(VkDevice         device,
                VkPhysicalDevice physicalDevice,
                int              width,
                int              height,
                Skin*            skin,
                VkSwapchainKHR   swapChain,
                VkCommandBuffer  commandBuffer,
                VkImage*         presentImages,
                VkFramebuffer*   frameBuffers,
                VkRenderPass     renderPass,
                VkDescriptorSet  descriptorSet,
                VkPipelineLayout pipelineLayout,
                VkPipeline       pipeline);

















