/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***

     Vulkan Graphics API
     Introductory Tutorials

     Minimal working examples (MWE) to build upon and 
     modify to gain an understanding of the API

/*
/****************************************************************/


#version 420

#define MAX_BONES             50

#define MAX_JOINT_WEIGHTS     4

layout (location = 0) in vec4   inPos;
layout (location = 1) in vec3   inNormal;
layout (location = 2) in ivec4  inBoneIndex; //[MAX_JOINT_WEIGHTS];
layout (location = 3) in vec4   inBoneWeight;//[MAX_JOINT_WEIGHTS];

layout (location = 0) out vec3  outNormal;
layout (location = 1) out vec3  outColor;

layout (binding = 0) uniform UBO 
{
    mat4 model;
    mat4 view;
    mat4 projection;
    mat4 bones[MAX_BONES];
} ubo;

void main() 
{
  mat4 boneTransform = ubo.bones[inBoneIndex[0]] * inBoneWeight[0];
  boneTransform     += ubo.bones[inBoneIndex[1]] * inBoneWeight[1];
  boneTransform     += ubo.bones[inBoneIndex[2]] * inBoneWeight[2];
  boneTransform     += ubo.bones[inBoneIndex[3]] * inBoneWeight[3];
  
  gl_Position = ubo.projection * ubo.view * ubo.model * boneTransform * vec4(inPos.xyz, 1.0);
  
  outNormal    = (boneTransform * vec4(inNormal.xyz, 0)).xyz;	
  outNormal	 = normalize(outNormal);
  
  // Debug - set the color based on the weights and the active
  // bones getting used
  outColor = vec3(0,0,0);
  if ( inBoneIndex[0] == 0 ) outColor += vec3(1,0,0) * inBoneWeight[0];
  if ( inBoneIndex[0] == 1 ) outColor += vec3(0,1,0) * inBoneWeight[0];
  if ( inBoneIndex[0] == 2 ) outColor += vec3(0,0,1) * inBoneWeight[0];
  
  if ( inBoneIndex[1] == 0 ) outColor += vec3(1,0,0) * inBoneWeight[1];
  if ( inBoneIndex[1] == 1 ) outColor += vec3(0,1,0) * inBoneWeight[1];
  if ( inBoneIndex[1] == 2 ) outColor += vec3(0,0,1) * inBoneWeight[1];

}// End main(..)
